@@ -40,24 +40,30 @@
 #define TOC_OFFSET 150
 #define SECTORS_PER_SEC 75
 #define POLL_TIMEOUT 1000
+#define REFRESH_RATE 5
 
 struct _GooPlayerCDPrivateData {
-	GooCdrom     *cdrom;
-	GstElement   *play_thread;
-	GstElement   *source;
-	GstPad       *source_pad;
-	GstElement   *volume;
-	gint64        total_time;
-	gint64        n_tracks;
-	GstFormat     track_format, sector_format;
-	GList        *tracks;
-	TrackInfo    *current_track;
-	int           tick;
-	char         *discid;
-	char         *artist;
-	char         *title;
-	char         *genre;
-	guint         update_state_id;
+	GooCdrom        *cdrom;
+	GstElement      *play_thread;
+	GstElement      *source;
+	GstPad          *source_pad;
+	GstElement      *volume;
+	gint64           total_time;
+	gint64           n_tracks;
+	GstFormat        track_format, sector_format;
+	GList           *tracks;
+	TrackInfo       *current_track;
+	int              tick;
+	char            *discid;
+	char            *artist;
+	char            *title;
+	char            *genre;
+	guint            update_state_id;
+
+	GThread         *thread;
+	GMutex          *yes_or_no;
+	guint            check_id;
+	gboolean         exiting;
 
 	CDDBSlaveClient *cddb_client;
 };
@@ -98,7 +104,8 @@ goo_player_cd_get_type ()
 
 
 static void
-action_done (GooPlayer *player, GooPlayerAction action)
+action_done (GooPlayer       *player, 
+	     GooPlayerAction  action)
 {
 	g_signal_emit_by_name (G_OBJECT (player), "done", action, NULL);
 }
@@ -286,27 +293,85 @@ cd_player_update (GooPlayer *player)
 }
 
 
-static void
-cd_player_list (GooPlayer *player)
+static int
+check_thread (gpointer data)
 {
+	GooPlayer              *player = data;
+	GooPlayerCD            *player_cd = GOO_PLAYER_CD (player);
+	GooPlayerCDPrivateData *priv = player_cd->priv;
+	GString                *offsets;
+	GList                  *scan;
+	gboolean                done, exiting;
+
+	/* Remove check. */
+
+        g_source_remove (priv->check_id);
+        priv->check_id = 0;
+	
+	/**/
+
+	g_mutex_lock (priv->yes_or_no);
+	done = priv->thread == NULL;
+	exiting = priv->exiting;
+        g_mutex_unlock (priv->yes_or_no);
+
+	if (exiting) {
+		goo_player_set_is_busy (player, FALSE);
+		destroy_pipeline (player_cd, TRUE);
+		return FALSE;
+	}
+
+	if (!done) {
+		priv->check_id = g_timeout_add (REFRESH_RATE, 
+						check_thread, 
+						player);
+		return FALSE;
+	}
+
+	/**/
+
+	goo_player_set_is_busy (player, FALSE);
+	gst_element_set_state (priv->play_thread, GST_STATE_NULL);
+	goo_player_set_state (player, GOO_PLAYER_STATE_STOPPED, TRUE);
+	action_done (player, GOO_PLAYER_ACTION_LIST);
+	destroy_pipeline (player_cd, TRUE);
+
+	/**/
+
+	offsets = g_string_new (NULL);
+	
+	for (scan = priv->tracks; scan; scan = scan->next) {
+		TrackInfo *track = scan->data;
+		char      *offset;
+		offset = g_strdup_printf ("%" G_GINT64_FORMAT " ", track->from_sector);
+		g_string_append (offsets, offset);
+		g_free (offset);
+	}
+
+	cddb_slave_client_query (priv->cddb_client, 
+				 priv->discid, 
+				 priv->n_tracks, 
+				 offsets->str, 
+				 priv->total_time / GST_SECOND,
+				 PACKAGE,
+				 VERSION);
+
+	g_string_free (offsets, TRUE);
+
+	return FALSE;
+}
+
+
+static void * 
+list_thread (void *thread_data)
+{
+	GooPlayer              *player = thread_data;
 	GooPlayerCD            *player_cd = GOO_PLAYER_CD (player);
 	GooPlayerCDPrivateData *priv = player_cd->priv;
 	GstFormat               time_format = GST_FORMAT_TIME;
 	gint                    i;
 	gint64                  from_sector = 0, total_sectors;
-	GString                *offsets;
-	GList                  *scan;
 
-	goo_player_cd_empty_list (player);
-
-	if (goo_cdrom_get_state (priv->cdrom) != GOO_CDROM_STATE_OK) {
-		debug (DEBUG_INFO, "NOT OK\n");
-		goo_player_set_state (player, GOO_PLAYER_STATE_ERROR, TRUE);
-		action_done (player, GOO_PLAYER_ACTION_LIST);
-		return;
-	}
-
-	create_pipeline (player_cd);
 	gst_element_set_state (priv->play_thread, GST_STATE_PAUSED);
 
 	/**/
@@ -354,32 +419,39 @@ cd_player_list (GooPlayer *player)
 	g_object_get (priv->source, "discid", &(priv->discid), NULL);
 	debug (DEBUG_INFO, "DISC ID: %s\n", priv->discid);
 
-	gst_element_set_state (priv->play_thread, GST_STATE_NULL);
-	goo_player_set_state (player, GOO_PLAYER_STATE_STOPPED, TRUE);
-	action_done (player, GOO_PLAYER_ACTION_LIST);
-	destroy_pipeline (player_cd, TRUE);
+	g_mutex_lock (priv->yes_or_no);
+	priv->thread = NULL;
+	g_mutex_unlock (priv->yes_or_no);
 
-	/**/
+	g_thread_exit (NULL);
 
-	offsets = g_string_new (NULL);
-	
-	for (scan = priv->tracks; scan; scan = scan->next) {
-		TrackInfo *track = scan->data;
-		char      *offset;
-		offset = g_strdup_printf ("%" G_GINT64_FORMAT " ", track->from_sector);
-		g_string_append (offsets, offset);
-		g_free (offset);
+	return NULL;
+}
+
+
+static void
+cd_player_list (GooPlayer *player)
+{
+	GooPlayerCD            *player_cd = GOO_PLAYER_CD (player);
+	GooPlayerCDPrivateData *priv = player_cd->priv;
+
+	if (goo_player_get_is_busy (player))
+		return;
+
+	goo_player_cd_empty_list (player);
+
+	if (goo_cdrom_get_state (priv->cdrom) != GOO_CDROM_STATE_OK) {
+		debug (DEBUG_INFO, "NOT OK\n");
+		goo_player_set_state (player, GOO_PLAYER_STATE_ERROR, TRUE);
+		action_done (player, GOO_PLAYER_ACTION_LIST);
+		return;
 	}
 
-	cddb_slave_client_query (priv->cddb_client, 
-				 priv->discid, 
-				 priv->n_tracks, 
-				 offsets->str, 
-				 priv->total_time / GST_SECOND,
-				 PACKAGE,
-				 VERSION);
+	goo_player_set_is_busy (player, TRUE);
+	create_pipeline (player_cd);
+	priv->thread  = g_thread_create (list_thread, player, FALSE, NULL);
 
-	g_string_free (offsets, TRUE);
+	priv->check_id = g_timeout_add (REFRESH_RATE, check_thread, player);
 }
 
 
@@ -392,6 +464,9 @@ cd_player_seek_song (GooPlayer *player,
 	GstEvent    *event;
 	int          track_n;
 
+	if (goo_player_get_is_busy (player))
+		return;
+
 	if (priv->n_tracks == 0) {
 		action_done (player, GOO_PLAYER_ACTION_SEEK_SONG);
 		return;
@@ -462,6 +537,9 @@ cd_player_skip_to (GooPlayer *player,
 	gint64       from_sector, to_sector, sectors;
 	GstEvent    *event;
 
+	if (goo_player_get_is_busy (player))
+		return;
+
 	if (priv->play_thread == NULL)
 		return;
 
@@ -484,6 +562,9 @@ cd_player_play (GooPlayer *player)
 	GooPlayerCDPrivateData *priv = player_cd->priv;
 	int vol;
 
+	if (goo_player_get_is_busy (player))
+		return;
+
 	if (priv->n_tracks == 0) {
 		action_done (player, GOO_PLAYER_ACTION_PLAY);
 		return;
@@ -509,6 +590,9 @@ cd_player_pause (GooPlayer *player)
 	GooPlayerCD *player_cd = GOO_PLAYER_CD (player);
 	GooPlayerCDPrivateData *priv = player_cd->priv;
 
+	if (goo_player_get_is_busy (player))
+		return;
+
 	if (priv->play_thread == NULL)
 		return;
 
@@ -524,6 +608,9 @@ cd_player_stop (GooPlayer *player)
 	GooPlayerCD *player_cd = GOO_PLAYER_CD (player);
 	GooPlayerCDPrivateData *priv = player_cd->priv;
 
+	if (goo_player_get_is_busy (player))
+		return;
+
 	if (priv->play_thread == NULL)
 		return;
 
@@ -541,6 +628,9 @@ cd_player_get_song_list (GooPlayer *player)
 	GooPlayerCDPrivateData *priv = player_cd->priv;
 	GList *list = NULL, *scan;
 
+	if (goo_player_get_is_busy (player))
+		return NULL;
+
 	for (scan = priv->tracks; scan; scan = scan->next) {
 		TrackInfo *track = scan->data;
 		list = g_list_prepend (list, create_song_info (player_cd, track));
@@ -558,6 +648,9 @@ cd_player_eject (GooPlayer *player)
 	gboolean      result;
 	GooCdromState cdrom_state;
 
+	if (goo_player_get_is_busy (player))
+		return FALSE;
+
 	destroy_pipeline (player_cd, TRUE);
 
 	cdrom_state = goo_cdrom_get_state (priv->cdrom);
@@ -581,6 +674,9 @@ cd_player_set_location (GooPlayer  *player,
 {
 	GooPlayerCD *player_cd = GOO_PLAYER_CD (player);
 
+	if (goo_player_get_is_busy (player))
+		return TRUE;
+
 	debug (DEBUG_INFO, "LOCATION: %s\n", location);
 
 	destroy_pipeline (player_cd, FALSE);
@@ -602,6 +698,10 @@ cd_player_set_volume (GooPlayer  *player,
 		      int         vol)
 {
 	GooPlayerCDPrivateData *priv = GOO_PLAYER_CD (player)->priv;
+
+	if (goo_player_get_is_busy (player))
+		return;
+
 	if (priv->volume != NULL)
 		gst_mixer_set_volume (GST_MIXER (priv->volume), NULL, &vol);
 }
@@ -775,6 +875,10 @@ goo_player_cd_init (GooPlayerCD *player)
 	player->priv = g_new0 (GooPlayerCDPrivateData, 1);
 	priv = player->priv;
 
+	priv->yes_or_no = g_mutex_new ();
+	priv->check_id = 0;
+	priv->exiting = FALSE,
+
 	priv->discid = NULL;
 	priv->n_tracks = 0;
 	priv->tracks = NULL;
@@ -802,12 +906,22 @@ goo_player_cd_finalize (GObject *object)
   
 	player_cd = GOO_PLAYER_CD (object);
 	player = GOO_PLAYER (object);
+	
+	g_mutex_lock (player_cd->priv->yes_or_no);
+	player_cd->priv->exiting = TRUE;
+        g_mutex_unlock (player_cd->priv->yes_or_no);
 
-	if (player_cd->priv != NULL) {
+ 	if (player_cd->priv != NULL) {
 		GooPlayerCDPrivateData *priv = player_cd->priv;
+		
+		if (priv->check_id != 0) {
+			g_source_remove (priv->check_id);
+			priv->check_id = 0;
+		}
 
-		if (goo_player_get_state (player) != GOO_PLAYER_STATE_STOPPED)
-			goo_player_stop (player);
+		destroy_pipeline (player_cd, FALSE);
+
+		g_mutex_free (priv->yes_or_no);
 
 		remove_state_polling (player_cd);
 		destroy_pipeline (player_cd, FALSE);
@@ -853,7 +967,11 @@ goo_player_cd_new (const char *location)
 const char *
 goo_player_cd_get_discid (GooPlayerCD *player)
 {
-	return player->priv->discid;
+	/*
+	if (goo_player_get_is_busy (GOO_PLAYER (player)))
+		return NULL;
+		else*/
+		return player->priv->discid;
 }
 
 
@@ -863,6 +981,11 @@ goo_player_cd_get_track (GooPlayerCD *player,
 {
 	GList *scan;
 
+	/*
+	if (goo_player_get_is_busy (GOO_PLAYER (player)))
+		return NULL;
+	*/
+
 	for (scan = player->priv->tracks; scan; scan = scan->next) {
 		TrackInfo *track = scan->data;
 		if (track->number == n)
@@ -876,26 +999,42 @@ goo_player_cd_get_track (GooPlayerCD *player,
 GList *
 goo_player_cd_get_tracks (GooPlayerCD *player)
 {
-	return track_list_dup (player->priv->tracks);
+	/*
+	if (goo_player_get_is_busy (GOO_PLAYER (player)))
+		return NULL;
+		else*/
+		return track_list_dup (player->priv->tracks);
 }
 
 
 const char *
 goo_player_cd_get_artist (GooPlayerCD *player)
 {
-	return player->priv->artist;
+	/*
+	if (goo_player_get_is_busy (GOO_PLAYER (player)))
+		return NULL;
+		else*/
+		return player->priv->artist;
 }
 
 
 const char *
-goo_player_cd_get_album  (GooPlayerCD *player)
+goo_player_cd_get_album (GooPlayerCD *player)
 {
-	return player->priv->title;
+	/*
+	if (goo_player_get_is_busy (GOO_PLAYER (player)))
+		return NULL;
+		else*/
+		return player->priv->title;
 }
 
 
 const char *
-goo_player_cd_get_genre  (GooPlayerCD *player)
+goo_player_cd_get_genre (GooPlayerCD *player)
 {
-	return player->priv->genre;
+	/*
+	if (goo_player_get_is_busy (GOO_PLAYER (player)))
+		return NULL;
+		else*/
+		return player->priv->genre;
 }
