@@ -38,6 +38,9 @@
 #include "goo-application.h"
 #include "gtk-utils.h"
 
+#ifdef HAVE_NEON
+#include <ne_socket.h>
+#endif /* HAVE_NEON*/
 
 static void     prepare_app         (poptContext pctx);
 static void     initialize_data     (void);
@@ -50,8 +53,13 @@ static BonoboObject *goo_application = NULL;
 
 GtkWindow *main_window = NULL;
 
+static char *default_device = NULL;
+
 
 struct poptOption options[] = {
+	{ "device", 'd', POPT_ARG_STRING, &default_device, 0,
+	  N_("CD device to be used"),
+	  N_("DEVICE_PATH") },
 	{ NULL, '\0', 0, NULL, 0 }
 };
 
@@ -106,12 +114,10 @@ int main (int argc, char **argv)
 		exit (0);
 	}
 
-	/*
 	if (! g_thread_supported ()) {
 		g_thread_init (NULL);
 		gdk_threads_init ();
 	}
-	*/
 
 	if (! gnome_vfs_init ()) 
                 g_error ("Cannot initialize the Virtual File System.");
@@ -121,6 +127,10 @@ int main (int argc, char **argv)
 	gst_init (NULL, NULL);
 	/*gst_use_threads (TRUE);*/
 
+#ifdef HAVE_NEON
+	ne_sock_init ();
+#endif /* HAVE_NEON*/
+
 	goo_stock_init ();
 	init_session (argv);
 	initialize_data ();
@@ -141,14 +151,14 @@ int main (int argc, char **argv)
 static void 
 initialize_data ()
 {
-	char *icon_path = PIXMAPSDIR "/goo.png";
+	char *icon_path = PIXMAPSDIR "/goobox.png";
 
 	if (! path_is_file (icon_path))
                 g_warning ("Could not find %s", icon_path);
 	else
 		gnome_window_icon_set_default_from_file (icon_path);
 
-	eel_gconf_monitor_add ("/apps/goo");
+	eel_gconf_monitor_add ("/apps/goobox");
 }
 
 
@@ -199,15 +209,13 @@ prepare_app (poptContext pctx)
 		return;
 	}
 
-	goo_application = goo_application_new (gdk_screen_get_default ());
-
-	if (session_is_restored ()) {
+	if (session_is_restored ()) 
 		load_session ();
-		return;
-	}
-
-	main_window = goo_window_new ();
+	else 
+		main_window = goo_window_new (default_device);
 	gtk_widget_show (GTK_WIDGET (main_window));
+
+	goo_application = goo_application_new (gdk_screen_get_default ());
 }
 
 
@@ -224,6 +232,15 @@ static const char *program_argv0 = NULL;
 static void
 save_session (GnomeClient *client)
 {
+	const char  *prefix;
+
+	prefix = gnome_client_get_config_prefix (client);
+	gnome_config_push_prefix (prefix);
+
+	gnome_config_set_string ("Session/location", goo_player_get_location (goo_window_get_player (GOO_WINDOW (main_window))));
+
+	gnome_config_pop_prefix ();
+	gnome_config_sync ();
 }
 
 
@@ -312,5 +329,15 @@ session_is_restored (void)
 gboolean
 load_session (void)
 {
+	char *location;
+
+	gnome_config_push_prefix (gnome_client_get_config_prefix (master_client));
+
+	location = gnome_config_get_string ("Session/location");
+	main_window = goo_window_new (location);
+	g_free (location);
+
+	gnome_config_pop_prefix ();
+
 	return TRUE;
 }
